/*  ___   ___  ___  _   _  ___   ___   ____ ___  ____  
 * / _ \ /___)/ _ \| | | |/ _ \ / _ \ / ___) _ \|    \ 
 *| |_| |___ | |_| | |_| | |_| | |_| ( (__| |_| | | | |
 * \___/(___/ \___/ \__  |\___/ \___(_)____)___/|_|_|_|
 *                  (____/ 
 * OSOYOO IoT Tutorial: Use OSOYOO IoT APP to control a servo remotely
 * Tutorial link: https://osoyoo.com/?p=52937
 * CopyRIGHT www.osoyoo.com
*/


#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <Servo.h>
Servo myservo;  // create servo object to control a servo

#ifndef STASSID
#define STASSID "***" // replace *** with your wifi SSID
#define STAPSK  "***" // replace *** with your wifi password
#endif

unsigned int localPort = 8888;      // local port to listen on

// buffers for receiving and sending data
char packetBuffer[UDP_TX_PACKET_MAX_SIZE + 1]; //buffer to hold incoming packet,
String msg = "";       // a string to send back

WiFiUDP Udp;

void setup() {
  myservo.attach(D1);  // attaches the servo on pin D1 to the servo object

  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(STASSID, STAPSK);
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print('.');
    delay(500);
  }
  Serial.print("Connected! IP address: ");
  Serial.println(WiFi.localIP());
  Serial.printf("UDP server on port %d\n", localPort);
  Udp.begin(localPort);
}

void loop() {
  // if there's data available, read a packet
  int packetSize = Udp.parsePacket();
  if (packetSize) {
    Serial.printf("Received packet of size %d from %s:%d\n    (to %s:%d, free heap = %d B)\n",
                  packetSize,
                  Udp.remoteIP().toString().c_str(), Udp.remotePort(),
                  Udp.destinationIP().toString().c_str(), Udp.localPort(),
                  ESP.getFreeHeap());

    // read the packet into packetBufffer
    int n = Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);
    packetBuffer[n] = 0;
    Serial.println("Contents:");
    Serial.println(packetBuffer);
    if (packetBuffer[0]=='F')
    {
     myservo.write(90);// tell servo to go to position 90
     Serial.println("Turn to 90 degree" );
     msg = "Turn to 90 degree\r\n";
    }
    if (packetBuffer[0]=='G')
    {
     myservo.write(180);// tell servo to go to position 180
     Serial.println("Turn to 180 degree" );
     msg = "Turn to 180 degree\r\n";
    }
    if (packetBuffer[0]=='H')
    {
     myservo.write(0);// tell servo to go to position 0
     Serial.println("Turn to 0 degree" );
     msg = "Turn to 0 degree\r\n";
    }
    
    // send a reply, to the IP address and port that sent us the packet we received
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    Udp.write(msg.c_str());
    Udp.endPacket();
  }
}
